(herald "Needham-Schroeder Lowe Public-Key Protocol"
	(comment "This protocol eliminates the man-in-the-middle"
		 "attack discovered by Gavin Lowe."))

;;; Used to generate output for inclusion in the primer.
;;; Use margin = 60 (-m 60) to generate the output.

(defprotocol ns basic
  (defrole init
    (vars (my_init_id yr_resp_id name) (n1 n2 text))
    (trace
     (send (enc n1 my_init_id (pubk yr_resp_id)))
     (recv (enc n1 n2 yr_resp_id (pubk my_init_id)))
     (send (enc n2 (pubk yr_resp_id)))))
  (defrole resp
    (vars (my_resp_id yr_init_id name) (n2 n1 text))
    (trace
     (recv (enc n1 yr_init_id (pubk my_resp_id)))
     (send (enc n1 n2 my_resp_id (pubk yr_init_id)))
     (recv (enc n2 (pubk my_resp_id)))))
  (comment "Needham-Schroeder"))

;;; The initiator point-of-view
(defskeleton ns
  (vars (a b name) (n1 text))
  (defstrand init 3 (my_init_id a) (yr_resp_id b) (n1 n1))
  (non-orig (privk b) (privk a))
  (uniq-orig n1)
  (comment "Initiator point-of-view"))

;;; The responder point-of-view
(defskeleton ns
  (vars (a b name) (n2 text))
  (defstrand resp 3 (yr_init_id a) (my_resp_id b) (n2 n2))
  (non-orig (privk a))
  (uniq-orig n2)
  (comment "Responder point-of-view"))

(defskeleton ns
  (vars (a b name) (n1 n2 text))
  (defstrand resp 3 (yr_init_id a) (my_resp_id b) (n2 n2) (n1 n1))
  (deflistener n2)
  (non-orig (privk a) (privk b))
  (uniq-orig n1 n2)
  (comment "Responder point-of-view, secrecy?"))

(defskeleton ns
  (vars (a b a1 b1 name) (n1 n2 text))
  (defstrand resp 3 (yr_init_id a) (my_resp_id b) (n1 n1) (n2 n2))
  (defstrand init 3 (my_init_id a1) (yr_resp_id b1) (n1 n1) (n2 n2))
  (non-orig (privk a) (privk b))
  (uniq-orig n1 n2)
  (comment "Implicit auth, both nonces u.o.")) 

