(herald "Second approx to TLS:  First get cert, then send pms encrypted.")


(defmacro (hash x) 
  (enc "hash_zero" x))

(defmacro (cert name public_key ca_sig_key) 
  (cat name public_key 
       (enc (hash (cat "cert" name public_key)) 
	    ca_sig_key)))

(defmacro (client_key_exch pm key)
  (enc "client_version" pm key))

(defprotocol tls1 basic 
  (defrole client 
    (vars (pre_master_secret skey) (server_key akey) (ca server_name name))
    (trace 
     (recv (cert server_name server_key (privk ca)))
     (send 
      (client_key_exch pre_master_secret server_key)))
    (non-orig (privk ca)))
  (defrole server 
    (vars (pre_master_secret skey) (server_key akey))
    (trace 
     (recv 
      (client_key_exch pre_master_secret server_key))))
  (defrole certificate_auth
    (vars (subject_name ca name))
    (trace 
     (send (cert subject_name (pubk subject_name) (privk ca))))))




(defskeleton tls1
  (vars (pms skey) (k akey) (ca s name)) 
  (defstrand client 2 (pre_master_secret pms) (server_key k) (ca ca) (server_name s))
  (deflistener pms)
  (non-orig (invk k))
  (uniq-orig pms)
  (comment "Confidentiality should be OK."))

(defskeleton tls1
  (vars (pms skey) (k akey) (ca s name)) 
  (defstrand client 2 (pre_master_secret pms) (server_key k) (ca ca) (server_name s))
  (non-orig (invk k))
  (uniq-orig pms)
  (comment "Authenticates CA."))

(defskeleton tls1
  (vars (pms skey) (k akey) (ca s name)) 
  (defstrand client 2 (pre_master_secret pms) (server_key k) (ca ca) (server_name s))
  (defstrand server 1 (pre_master_secret pms))
  (non-orig (invk k))
  (uniq-orig pms)
  (comment "Authenticates CA and implicit auth for server."))

(defskeleton tls1
  (vars (pms skey) (k akey)) 
  (defstrand server 1 (pre_master_secret pms) (server_key k))
  (deflistener pms)
  (non-orig (invk k))
  (uniq-orig pms)
  (comment "Neither confidentiality nor authentication guarantees to server"))
