(herald "Fourth approx to TLS:  Both parties get cert, then c sends pms encrypted and hash.  CA guarantees privk uncompromised."
	(bound 14))


(defmacro (hash x) 
  (enc "hash_zero" x))

(defmacro (cert name public_key ca_sig_key) 
  (cat name public_key 
       (enc (hash (cat "cert" name public_key)) 
	    ca_sig_key)))

(defmacro (client_key_exch pm key)
  (enc "client_version" pm key))

(defprotocol tls3 basic 
  (defrole client 
    (vars (pre_master_secret skey) 
	  (server_key client_key akey)
	  (ca server_name client_name name))
    (trace 
     (recv (cert server_name server_key (privk ca)))
     (recv (cert client_name client_key (privk ca)))
     (send 
      (client_key_exch pre_master_secret server_key))
     (send (enc (hash (cat (cert server_name server_key (privk ca))
			   (cert client_name client_key (privk ca))
			   (client_key_exch pre_master_secret server_key)))
		(invk client_key))))
    (non-orig (privk ca)))

  (defrole server 
    (vars (pre_master_secret skey) 
	  (server_key client_key akey) 
	  (ca server_name client_name name))
    (trace 
     (recv (cert server_name server_key (privk ca)))
     (recv (cert client_name client_key (privk ca)))
     (recv 
      (client_key_exch pre_master_secret server_key))
     (recv (enc (hash (cat (cert server_name server_key (privk ca))
			   (cert client_name client_key (privk ca))
			   (client_key_exch pre_master_secret server_key)))
		(invk client_key))))
    (non-orig (privk ca)))
  (defrole certificate_auth
    (vars (subject_name ca name))
    (trace 
     (send (cert subject_name (pubk subject_name) (privk ca))))
    (non-orig (privk subject_name))))




(defskeleton tls3
  (vars (pms skey) (k akey) (ca s name)) 
  (defstrand client 4 (pre_master_secret pms) (server_key k) (ca ca) (server_name s))
  (deflistener pms)
  (non-orig (invk k))
  (uniq-orig pms)
  (comment "Confidentiality should be OK."))


(defskeleton tls3
  (vars (pms skey) (k akey) (ca s name)) 
  (defstrand client 4 (pre_master_secret pms) (server_key k) (ca ca) (server_name s))
  (non-orig (invk k))
  (uniq-orig pms)
  (comment "Authenticates CA."))

(defskeleton tls3
  (vars (pms skey) (k akey) (ca s c name)) 
  (defstrand client 4 (pre_master_secret pms) (server_key k) 
    (client_name c) (server_name s))
  (defstrand server 4 (pre_master_secret pms) (ca ca))
  (non-orig (invk k) (privk c))
  (uniq-orig pms)
  (comment "Authenticates CA and implicit auth for server."))

(defskeleton tls3 
  (vars (pms skey) (k akey)) 
  (defstrand server 4 (pre_master_secret pms) (server_key k))
  (deflistener pms)
  (non-orig (invk k))
  (uniq-orig pms)
  (comment "Confidentiality guarantees to server"))

(defskeleton tls3 
  (vars (pms skey) (k akey)) 
  (defstrand server 4 (pre_master_secret pms) (server_key k))
  (non-orig (invk k))
  (uniq-orig pms)
  (comment "Auth guarantees to server"))

(defskeleton tls3 
  (vars (pms skey) (k akey)) 
  (defstrand server 4 (pre_master_secret pms) (server_key k))
  (defstrand server 4 (pre_master_secret pms) (server_key k))
  (non-orig (invk k))
  (uniq-orig pms)
  (comment "Session property?")
  (comment "No.  See 232."))
