(herald "Fifth approx to TLS:  Both parties get cert, then c sends pms encrypted and hash.  CA guarantees privk uncompromised.  Session nonces used."
	(bound 14))


(defmacro (hash x) 
  (enc "hash_zero" x))

(defmacro (cert name public_key ca_sig_key) 
  (cat name public_key 
       (enc (hash (cat "cert" name public_key)) 
	    ca_sig_key)))

(defmacro (client_key_exch pm key)
  (enc "client_version" pm key))

(defprotocol tls4 basic 
  (defrole client 
    (vars (pre_master_secret skey) 
	  (server_key client_key akey)
	  (ca server_name client_name name)
	  (ns nc text))
    (trace 
     (send nc)
     (recv ns)
     (recv (cert server_name server_key (privk ca)))
     (recv (cert client_name client_key (privk ca)))
     (send 
      (client_key_exch pre_master_secret server_key))
     (send (enc nc ns 
		(hash (cat (cert server_name server_key (privk ca))
			   (cert client_name client_key (privk ca))
			   (client_key_exch pre_master_secret server_key)))
		(invk client_key))))
    (non-orig (privk ca)))

  (defrole server 
    (vars (pre_master_secret skey) 
	  (server_key client_key akey) 
	  (ca server_name client_name name)
	  (ns nc text))
    (trace 
     (recv nc)
     (send ns)
     (recv (cert server_name server_key (privk ca)))
     (recv (cert client_name client_key (privk ca)))
     (recv 
      (client_key_exch pre_master_secret server_key))
     (recv (enc nc ns
		(hash (cat (cert server_name server_key (privk ca))
			   (cert client_name client_key (privk ca))
			   (client_key_exch pre_master_secret server_key)))
		(invk client_key))))
    (non-orig (privk ca)))
  (defrole certificate_auth
    (vars (subject_name ca name))
    (trace 
     (send (cert subject_name (pubk subject_name) (privk ca))))
    (non-orig (privk subject_name))))




(defskeleton tls4
  (vars (pms skey) (k akey) (ca s name)) 
  (defstrand client 6 (pre_master_secret pms) (server_key k) (ca ca) (server_name s))
  (deflistener pms)
  (non-orig (invk k))
  (uniq-orig pms)
  (comment "Confidentiality should be OK."))


(defskeleton tls4
  (vars (pms skey) (k akey) (ca s name)) 
  (defstrand client 6 (pre_master_secret pms) (server_key k) (ca ca) (server_name s))
  (non-orig (invk k))
  (uniq-orig pms)
  (comment "Authenticates CA."))

(defskeleton tls4
  (vars (pms skey) (k akey) (ca s c name)) 
  (defstrand client 6 (pre_master_secret pms) (server_key k) 
    (client_name c) (server_name s))
  (defstrand server 6 (pre_master_secret pms) (ca ca))
  (non-orig (invk k) (privk c))
  (uniq-orig pms)
  (comment "Authenticates CA and implicit auth for server."))

(defskeleton tls4 
  (vars (pms skey) (k akey)) 
  (defstrand server 6 (pre_master_secret pms) (server_key k))
  (deflistener pms)
  (non-orig (invk k))
  (uniq-orig pms)
  (comment "Confidentiality guarantees to server"))

(defskeleton tls4 
  (vars (pms skey) (k akey)) 
  (defstrand server 6 (pre_master_secret pms) (server_key k))
  (non-orig (invk k))
  (uniq-orig pms)
  (comment "Auth guarantees to server"))

(defskeleton tls4 
  (vars (pms skey) (k akey) (ns text)) 
  (defstrand server 6 (pre_master_secret pms) (ns ns) (server_key k))
  (defstrand server 6 (pre_master_secret pms))
  (non-orig (invk k))
  (uniq-orig pms ns)
  (comment "Session property?"))
